************************************************************************
C Program: Ising3.f
C Purpose: Using Monte Carlo integration to calculate the energy and
C magnetization for the 2D Ising model.  I'm taking J=1.  The output is 
C the energy and magnetization for every iteration.
C
C This program provides an example of running a code with no arguments.
C All of the input data is assumed to be in data files in the appropriate
C directory.
C For example, you would invoke it with the command line
C     a.out 
C Output is written to output3.dat.
************************************************************************

      include 'ran2.f'
      implicit none

      integer NITER,NTHERM,NLAT,NITER_MAX,NLAT_MAX
      parameter(NITER_MAX=100000,NTHERM=100)
      parameter(NLAT_MAX=500)

      real*8 spins(NLAT_MAX,NLAT_MAX),temp,energy,magnet
      real*8 sum_energy,sum_magnet,ran2
      real*8 engArray(NITER_MAX),magArray(NITER_MAX)
      integer iseed,i,j,iargc

      character*72 input1,input2,input3,input4,output
      input1='./input31.dat'
      input2='./input32.dat'
      input3='./input33.dat'
      input4='./input34.dat'
      output='./output3.dat'

      open(10,file=input1,status="old")
      open(20,file=input2,status="old")
      open(30,file=input3,status="old")
      open(40,file=input4,status="old")
      open(80,file=output,status="unknown")

      read(10,*) temp
      read(20,*) iseed
      read(30,*) NITER
      read(40,*) NLAT

*-----------------------------------------------------------------------
C Randomly set spins to start.
*-----------------------------------------------------------------------
      do i=1,NLAT
         do j=1,NLAT
            if(ran2(iseed).lt.0.5) then
               spins(i,j)=1.
            else
               spins(i,j)=-1.
            end if
         end do
      end do
      call obsv(spins,NLAT,energy,magnet)

*-----------------------------------------------------------------------
C Thermalize the configuration.
*-----------------------------------------------------------------------
      do i=1,NTHERM
         call monte(spins,temp,iseed,NLAT)
      end do

*-----------------------------------------------------------------------
C Run the calculation.
*-----------------------------------------------------------------------
      do i=1,NITER
         call monte(spins,temp,iseed,NLAT)
         call obsv(spins,NLAT,energy,magnet)
         sum_energy=sum_energy+energy
         sum_magnet=sum_magnet+magnet
         engArray(i)=energy
         magArray(i)=magnet

      end do
      do i=1,NITER
         write(80,*) engArray(i),magArray(i)
c         write(6,*) engArray(i),magArray(i)
      end do
c      print*,sum_energy/float(NITER),sum_magnet/float(NITER)
      
 100  continue
      end

************************************************************************
C Subroutine: monte
C Purpose: Sweeps through the lattice, updating spins with Monte Carlo.
************************************************************************
      subroutine monte(spins,temp,iseed,NLAT)
      implicit none
      real*8 spins(NLAT,NLAT)
      real*8 flipped,energy_old,energy_new,temp,sum_nn,delta,ratio,ran2

      integer i,j,im1,jm1,ip1,jp1,iseed,NLAT
     
      do i=1,NLAT
         im1=i-1
         ip1=i+1
         if(im1.le.0) im1=im1+NLAT
         if(ip1.ge.NLAT+1) ip1=ip1-NLAT
         do j=1,NLAT
            jm1=j-1
            jp1=j+1
            if(jm1.le.0) jm1=jm1+NLAT
            if(jp1.ge.NLAT+1) jp1=jp1-NLAT
            flipped=-spins(i,j)
            sum_nn=spins(ip1,j)+spins(im1,j)+spins(i,jp1)+spins(i,jm1)
            energy_new=flipped*sum_nn
            energy_old=spins(i,j)*sum_nn
            delta=(energy_new-energy_old)/temp
            if(delta.ge.0.0) then
               spins(i,j)=flipped
            else
               ratio=exp(delta)
               if(ratio.ge.ran2(iseed)) spins(i,j)=flipped
            end if
         end do
      end do

      return
      end

************************************************************************
C Subroutine obsv
C Purpose: Calculate the energy and the magnetization
************************************************************************
      subroutine obsv(spins,NLAT,energy,magnet)
      implicit none
      
      real*8 spins(NLAT,NLAT)
      real*8 energy,magnet
      integer nlat,i,ip1,j,jp1
      
      magnet=0.0
      energy=0.0
      do i=1,NLAT
         ip1=i+1
         if(ip1.ge.NLAT+1) ip1=ip1-NLAT
         do j=1,NLAT
            jp1=j+1
            if(jp1.ge.NLAT+1) jp1=jp1-NLAT
            magnet=magnet+spins(i,j)
            energy=energy+spins(i,j)*(spins(ip1,j)+spins(i,jp1))
         end do
      end do
      magnet=magnet/float(NLAT*NLAT)
      energy=energy/float(NLAT*NLAT)
      return
      end
      
      
C The following code constitutes the old way that I handled input and
C output.  I'm putting it here so that it's out of the way but I still
C have record of it.

